# -*- coding: utf-8 -*-

from datetime import date
from PIL import Image

from django import forms
from django.forms.widgets import SelectDateWidget
from django.core.validators import validate_email

from lib import forms_inp


CHOIX_GENRES = ((1, "Masculin"), (2, "Féminin"))
TAILLE_PHOTO = (116, 144)
RATIO_PHOTO = float(TAILLE_PHOTO[0]) / float(TAILLE_PHOTO[1])
DOMAINES_INP_NET = ["inpt.fr", "inp-net.eu.org", "inpt.party"]
INFOS_CONTACT_ADMIN = {
    "mailLocal": (
        "E-mail sur les serveurs INP-net",
        "Adresse",
        forms_inp.RestrictDomaineEmailValidator(
            restrict_domain=DOMAINES_INP_NET,
            message_domain=(
                "Enter a valid email adress with domain in the restricted list : " + ", ".join(DOMAINES_INP_NET)
            ),
        ),
    ),
    "mailAnnexe": ("E-mail annexe", "Adresse", validate_email),
}


class IterateurDate(object):
    def __iter__(self):
        annee = date.today().year
        return range(annee - 40, annee).__iter__()


class EleveForm(forms.Form):
    chg_photo = forms.FileField(label="Nouvelle photo", required=False)

    uid = forms_inp.ReadOnlyField(label="Login INP-net")
    loginTP = forms_inp.ReadOnlyField(label="Login TP")

    nom = forms.CharField(label="Nom")
    prenom = forms.CharField(label="Prénom")

    genre = forms.ChoiceField(choices=CHOIX_GENRES)
    birthdate = forms.DateField(label="Date de naissance", widget=SelectDateWidget(years=IterateurDate()))

    # Informations école/AE/INP-net
    promo = forms_inp.ReadOnlyField(label="Promo")
    ecole = forms_inp.ReadOnlyField(label="École")
    filiere = forms_inp.ReadOnlyField(label="Filière")
    inscritAE = forms.BooleanField(label="Inscrit à l’AE", required=False)
    inscritFrappe = forms.BooleanField(label="Inscrit sur la Frappe", required=False)
    inscritPassVieEtudiant = forms.BooleanField(label="Cotisant au PassVie Étudiant", required=False)
    finPassVieEtudiant = forms.IntegerField(label="Expiration PassVie Étudiant (si inscrit)", required=False)

    # Adresse mail attribuée par l’école
    mail = forms_inp.ReadOnlyField(label="Adresse e-mail BDE")
    mailEcole = forms.EmailField(label="Adresse e-mail de l’école")

    loginShell = forms.CharField(label="Shell")

    def orient_photo(self, image):
        try:
            # Extraction du dictionnaire des métadonnées EXIF
            exif = image._getexif()
        except AttributeError:
            return None

        # Vérification de l'existence de l'EXIF, même si l'image est un JPEG
        if exif is None:
            return None

        # Vérification si l'image est un JPEG et donc si elle a des EXIF
        try:
            # Extraction de la métadonnée EXIF orientation de la photo, qui a pour key 274
            orientation = exif[274]
        except KeyError:
            return None

        return orientation

    def clean_chg_photo(self):
        chg_photo = self.cleaned_data["chg_photo"]

        if chg_photo is None:
            return chg_photo

        try:
            image = Image.open(chg_photo)
            image.load()
        except IOError:
            raise forms.ValidationError("Le fichier envoyé n’est pas une " "image valide.")

        orientation = self.orient_photo(image)

        # Si l'image est tournée d'un quart de tour dans le sens horaire
        if orientation == 6:
            image = image.rotate(-90)
            # Si l'image est tournée d'un quart de tour dans le sens anti-horaire
        elif orientation == 8:
            image = image.rotate(90)
            # Si l'image est à l'envers, vous êtes une bite

        larg, haut = image.size
        larg_alt = int(round(haut * RATIO_PHOTO))
        haut_alt = int(round(larg / RATIO_PHOTO))

        if larg_alt <= larg:
            decoupe = (larg - larg_alt) / 2
            image = image.crop((decoupe, 0, larg - decoupe, haut))
        else:
            decoupe = (haut - haut_alt) / 2
            image = image.crop((0, decoupe, larg, haut - decoupe))

        image = image.resize(TAILLE_PHOTO, Image.ANTIALIAS)

        self.chg_photo_image = image

        return chg_photo
